/*
 * interrupt.c
 *
 * Copyright (C) 2006-2008 Atmark Techno, Inc.
 *
 */

#include <ctype.h>
#include "medium.h"
#include "slot.h"
#include "memmap.h"
#include "medium.h"
#include "xintc_l.h"
#ifdef __PPC__
#include "xpseudo_asm_gcc.h"
#include "xexception_l.h"
#endif
#if defined(XPAR_OPB_SIL00U_0_BASEADDR)
#include "opb_sil00u.h"
#define XPAR_OPB_SIL00_0_IP2INTC_IRPT_MASK XPAR_OPB_SIL00U_0_IP2INTC_IRPT_MASK
#define OPB_SIL00_mWriteReg                OPB_SIL00U_mWriteReg
#define XPAR_OPB_SIL00_0_BASEADDR          XPAR_OPB_SIL00U_0_BASEADDR
#define OPB_SIL00_INTR_IER_OFFSET          OPB_SIL00U_INTR_IER_OFFSET
#define OPB_SIL00_INTR_DIER_OFFSET         OPB_SIL00U_INTR_DIER_OFFSET
#define OPB_SIL00_INTR_DGIER_OFFSET        OPB_SIL00U_INTR_DGIER_OFFSET
#define OPB_SIL00_mReadReg                 OPB_SIL00U_mReadReg
#define OPB_SIL00_INTR_ISR_OFFSET          OPB_SIL00U_INTR_ISR_OFFSET
#define XPAR_SYSTEM_INTC_OPB_SIL00_0_IP2INTC_IRPT_INTR   XPAR_SYSTEM_INTC_OPB_SIL00U_0_IP2INTC_IRPT_INTR
#elif defined(XPAR_SIL_CNTLR_BASEADDR)
#include "xps_sil00.h"
#define XPAR_OPB_SIL00_0_IP2INTC_IRPT_MASK XPAR_SIL_CNTLR_IP2INTC_IRPT_MASK
#define OPB_SIL00_mWriteReg                XPS_SIL00_mWriteReg
#define XPAR_OPB_SIL00_0_BASEADDR          XPAR_SIL_CNTLR_BASEADDR
#define OPB_SIL00_INTR_IER_OFFSET          XPS_SIL00_INTR_IPIER_OFFSET
#define OPB_SIL00_INTR_DIER_OFFSET         XPS_SIL00_INTR_DIER_OFFSET
#define OPB_SIL00_INTR_DGIER_OFFSET        XPS_SIL00_INTR_DGIER_OFFSET
#define OPB_SIL00_mReadReg                 XPS_SIL00_mReadReg
#define OPB_SIL00_INTR_ISR_OFFSET          XPS_SIL00_INTR_IPISR_OFFSET
#define XPAR_SYSTEM_INTC_OPB_SIL00_0_IP2INTC_IRPT_INTR   XPAR_INTC_SYSTEM_SIL_CNTLR_IP2INTC_IRPT_INTR
#define XPAR_SYSTEM_INTC_BASEADDR          XPAR_INTC_SYSTEM_BASEADDR
#define XPAR_SYSTEM_INTC_DEVICE_ID         XPAR_INTC_SYSTEM_DEVICE_ID
#else
#include "opb_sil00.h"
#endif

int timer_interrupt_handler(void * baseaddr_p)
{
    unsigned int csr;

    Xuint32 IntrStatus;

    IntrStatus = OPB_SIL00_mReadReg(XPAR_OPB_SIL00_0_BASEADDR, OPB_SIL00_INTR_ISR_OFFSET);
    slot();
    OPB_SIL00_mWriteReg(XPAR_OPB_SIL00_0_BASEADDR, OPB_SIL00_INTR_ISR_OFFSET, IntrStatus);
}


static void NullHandler(void *DataPtr)
{
}

int interrupt_init(void)
{
    unsigned long index;
    unsigned int timer_count = 1;
    /*enable interrupt */
    XIntc_RegisterHandler(XPAR_SYSTEM_INTC_BASEADDR,XPAR_SYSTEM_INTC_OPB_SIL00_0_IP2INTC_IRPT_INTR,
      (XInterruptHandler)timer_interrupt_handler, (void *)0);

#ifdef __PPC__
    mtevpr(0xFFFF0000);
    for(index = XEXC_ID_FIRST; index < XEXC_ID_LAST + 1; index++){
       XExc_RegisterHandler(index, (XExceptionHandler)NullHandler, XNULL);
    }
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,(XExceptionHandler)XIntc_DeviceInterruptHandler,
    (void*)XPAR_SYSTEM_INTC_DEVICE_ID);
    XExc_mEnableExceptions(XEXC_NON_CRITICAL);
#else
    microblaze_enable_interrupts();
#endif
    /* Start the interrupt controller */
    XIntc_mMasterEnable(XPAR_SYSTEM_INTC_BASEADDR);
    /* Enable timer interrupt in the interrupt controller */
    XIntc_mEnableIntr(XPAR_SYSTEM_INTC_BASEADDR, XPAR_OPB_SIL00_0_IP2INTC_IRPT_MASK);
    /* start the intr sil */
     /* Enable all interrupt source from user logic.*/
    OPB_SIL00_mWriteReg(XPAR_OPB_SIL00_0_BASEADDR, OPB_SIL00_INTR_IER_OFFSET, 0x00000001);
     /* Set global interrupt enable.*/
    OPB_SIL00_mWriteReg(XPAR_OPB_SIL00_0_BASEADDR, OPB_SIL00_INTR_DGIER_OFFSET, INTR_GIE_MASK);
}

int interrupt_clean(void)
{
    unsigned long index;
    /* disable interrupt */
#ifdef __PPC__
    XExc_mDisableExceptions(XEXC_NON_CRITICAL);
    XExc_RemoveHandler(XEXC_ID_NON_CRITICAL_INT);
    for(index = XEXC_ID_FIRST; index < XEXC_ID_LAST + 1; index++){
       XExc_RemoveHandler(index);
    }
    mtevpr(0);
#else
    microblaze_disable_interrupts();
#endif
    XIntc_mMasterDisable(XPAR_SYSTEM_INTC_BASEADDR);
    /* Disable timer interrupt in the interrupt controller */
    XIntc_mDisableIntr(XPAR_SYSTEM_INTC_BASEADDR, XPAR_OPB_SIL00_0_IP2INTC_IRPT_MASK);
    /* stop the timer */
    OPB_SIL00_mWriteReg(XPAR_OPB_SIL00_0_BASEADDR, OPB_SIL00_INTR_IER_OFFSET, 0);
    OPB_SIL00_mWriteReg(XPAR_OPB_SIL00_0_BASEADDR, OPB_SIL00_INTR_DGIER_OFFSET, 0);
}

